# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE

from __future__ import annotations

from awkward.highlevel import Array

ByteBehavior = Array
CharBehavior = Array
ByteStringBehavior = Array
StringBehavior = Array
